"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
class Permission {
    constructor(permissions) {
        // checkPermission('Post.create', object_s, user, object2_s)
        this.checkPermission = (operation, objects, user, objects2) => {
            objects = lodash_1.castArray(objects);
            objects2 = lodash_1.castArray(objects2);
            return objects.every((v, i) => this._checkPermission(operation, v, user, objects2[i]));
        };
        this.permissions = permissions;
        this.permissionsObj = this.getPermissionsObj(permissions);
    }
    _checkPermission(operation, object, user, object2) {
        const action = operation.split(/\./)[1];
        if (object === null)
            return action === 'read';
        const permissions = this.findPermissions(operation);
        return Boolean(permissions.find(p => {
            if (p.authenticated && !user) {
                return false;
            }
            if (p.query && !p.query(object, user, object2)) {
                return false;
            }
            if (p.fields) {
                const input = action === 'update' ? object2 : object;
                lodash_1.forOwn(input, (v, k) => {
                    if (!p.fields.includes(k)) {
                        delete input[k];
                    }
                });
            }
            return true;
        }));
    }
    // findPermissions('Post.create') => permissions
    findPermissions(operation) {
        const [typeName, action] = operation.split(/\./);
        return lodash_1.flatten(['*', `${typeName}.*`, operation].map(v => this.permissionsObj[v] || []));
    }
    // -> [..] => {'*': [..], 'Post.*': [..], 'Post.create': [..]}
    getPermissionsObj(permissions) {
        const ret = {};
        permissions.forEach(permission => {
            permission.operations.forEach(operation => {
                ret[operation] = ret[operation] || [];
                ret[operation].push(permission);
            });
        });
        return ret;
    }
}
exports.default = Permission;
