'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _httpProxyMiddleware = require('http-proxy-middleware');

var _httpProxyMiddleware2 = _interopRequireDefault(_httpProxyMiddleware);

var _apiErrorHandler = require('api-error-handler');

var _apiErrorHandler2 = _interopRequireDefault(_apiErrorHandler);

var _createHttp = require('./createHttp');

var _createHttp2 = _interopRequireDefault(_createHttp);

var _authEmpty = require('./auth-empty');

var _authEmpty2 = _interopRequireDefault(_authEmpty);

var _vendor = require('./vendor');

var _http = require('http');

var _subscriptionsTransportWs = require('subscriptions-transport-ws');

var _graphql = require('graphql');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * startServer({
 *   namespace: 'admin'
 *   uploadsDir: 'uploads'
 *   context: {}
 *   db: new Db()              // class Db { userFind(where), userCreate(props), userUpdate(user, props), userSerialize(user) } 
 *   schema: new Schema([..])  // 'guserver-schema-mini'
 *   auth: new Auth(..)        // 'guserver-auth-magic'
 *   apis: express.Router()
 * })
 */

//import './wsserver'
const DEFAULT_OPTION = {
  namespace: '',
  uploadsDir: 'uploads',
  auth: new _authEmpty2.default(),
  apis: _express2.default.Router()
};

function startServer(...options) {
  options = options.map(v => Object.assign({}, DEFAULT_OPTION, v));
  options.forEach(v => {
    v.schema = v.schema.buildSchema();
    v.namespaceWithSuffix = v.namespace === '' ? '' : `${v.namespace}/`;
  });

  const PORT = process.env.PORT || 80;
  const app = (0, _express2.default)();
  const server = (0, _http.createServer)(app);

  app.use(_bodyParser2.default.json());
  app.use(_vendor.logging);

  options.forEach(option => {
    app.use(`/${option.namespace}`, (0, _createHttp2.default)(option));
  });

  // Testing web production build
  // TODO: add option in package.json to enable it
  //if (process.env.NODE_ENV !== 'production') {
  //app.use('/', proxy({target: 'localhost:5000'}))
  //}

  // status: 404, message | name type stack
  app.use((0, _apiErrorHandler2.default)());

  server.listen(PORT, () => {
    options.forEach(option => {
      console.log(`>> localhost:${PORT}/${option.namespaceWithSuffix}graphql`);
      new _subscriptionsTransportWs.SubscriptionServer({ execute: _graphql.execute, subscribe: _graphql.subscribe, schema: option.schema }, {
        server,
        path: `/${option.namespaceWithSuffix}graphql`
      });
    });
  });
}

exports.default = startServer;