import 'pdjs'
import * as async from 'async'
import { map, upperFirst } from 'lodash'
import connection from 'vendor/typeorm'
import * as models from 'models'
const FIXTURES = require(`${process.cwd()}/scripts/seeds.dev`)

async function main() {
  const conn = await connection
  await conn.dropDatabase()
  await conn.synchronize()

  const jobs = map(FIXTURES, (values, key: string) => {
    return async function() {
      console.log(`>> Load ${key}`)
      const Model = models[upperFirst(key)]
      const { tableName } = Model.getRepository().metadata
      await conn.transaction(async () => {
        await conn.query(`ALTER TABLE "${tableName}" DISABLE TRIGGER ALL`)
        await Model.createQueryBuilder()
          .insert()
          .values(values)
          .execute()
        await conn.query(`ALTER TABLE "${tableName}" ENABLE TRIGGER ALL`)
      })
    }
  })

  async.series(jobs, err => {
    if (err) {
      err = err.original ? err.original : err
      console.log(err)
    }
    conn.close()
  })
}

main()
